'use strict';

var catalogModel = require('@backstage/catalog-model');
var aws = require('@langchain/aws');
var openai = require('@langchain/openai');
var langgraph = require('@langchain/langgraph');
var prebuilt = require('@langchain/langgraph/prebuilt');
var messages = require('@langchain/core/messages');
var transform = require('./util/transform.cjs.js');
var modifier = require('./util/modifier.cjs.js');
var config = require('./config/config.cjs.js');
var langfuseLangchain = require('langfuse-langchain');

class LangGraphReactAgentType {
  constructor(llm, tools, agent, options) {
    this.llm = llm;
    this.tools = tools;
    this.agent = agent;
    this.prompt = options.prompt;
    this.langfuseConfig = options.langfuseConfig;
  }
  prompt;
  langfuseConfig;
  static async fromConfig(rootConfig, agentConfig, tools, logger) {
    const prompt = agentConfig.prompt;
    const sharedLangGraphConfig = config.readSharedLangGraphAgentConfig(rootConfig);
    const agentLangGraphConfig = config.readLangGraphAgentConfig(agentConfig.config);
    let agentModel;
    if (agentLangGraphConfig.bedrock) {
      agentModel = LangGraphReactAgentType.createBedrockModel(agentLangGraphConfig);
    } else if (agentLangGraphConfig.openai) {
      agentModel = LangGraphReactAgentType.createOpenAIModel(agentLangGraphConfig);
    } else {
      throw new Error("No agent model configured");
    }
    logger.info(
      `Instantiating langgraph-react agent ${agentConfig.name} using model '${agentModel.getName()}'`
    );
    const agentCheckpointer = new langgraph.MemorySaver();
    const agent = prebuilt.createReactAgent({
      llm: agentModel,
      tools,
      checkpointSaver: agentCheckpointer,
      messageModifier: agentLangGraphConfig.messagesMaxTokens ? async (e) => {
        return await messages.trimMessages(e, {
          maxTokens: agentLangGraphConfig.messagesMaxTokens,
          strategy: "last",
          tokenCounter: modifier.tiktokenCounter,
          includeSystem: true,
          startOn: "human"
        });
      } : void 0
    });
    return new LangGraphReactAgentType(agentModel, tools, agent, {
      prompt,
      langfuseConfig: sharedLangGraphConfig?.langfuse
    });
  }
  static createBedrockModel(config) {
    const { modelId, region } = config.bedrock;
    return new aws.ChatBedrockConverse({
      model: modelId,
      region,
      streaming: true,
      temperature: config.temperature,
      maxTokens: config.maxTokens,
      topP: config.topP
    });
  }
  static createOpenAIModel(config) {
    const { modelName, apiKey } = config.openai;
    return new openai.ChatOpenAI({
      apiKey,
      streaming: true,
      modelName,
      temperature: config.temperature,
      maxTokens: config.maxTokens,
      topP: config.topP
    });
  }
  buildCallbackHandler(sessionId, userId) {
    const callbacks = [];
    if (this.langfuseConfig) {
      callbacks.push(
        new langfuseLangchain.CallbackHandler({
          ...this.langfuseConfig,
          sessionId,
          userId
        })
      );
    }
    return callbacks;
  }
  async stream(userMessage, sessionId, newSession, userEntityRef, _, options) {
    const messages$1 = [];
    if (this.prompt) {
      if (newSession) {
        messages$1.push(this.buildSystemPrompt(this.prompt, userEntityRef));
      }
    }
    messages$1.push(new messages.HumanMessage(userMessage));
    const eventStreamFinalRes = this.agent.streamEvents(
      {
        messages: messages$1
      },
      {
        version: "v2",
        callbacks: this.buildCallbackHandler(
          sessionId,
          catalogModel.stringifyEntityRef(userEntityRef)
        ),
        configurable: {
          thread_id: sessionId,
          credentials: options.credentials
        }
      }
    );
    return eventStreamFinalRes.pipeThrough(
      new transform.ResponseTransformStream(sessionId)
    );
  }
  async generate(prompt, sessionId, userEntityRef, logger, options) {
    const messages$1 = [];
    if (this.prompt) {
      messages$1.push(this.buildSystemPrompt(this.prompt, userEntityRef));
    }
    messages$1.push(new messages.HumanMessage(prompt));
    let output = "";
    const agent = prebuilt.createReactAgent({
      llm: this.llm,
      tools: this.tools
    });
    const finalState = await agent.invoke(
      { messages: messages$1 },
      {
        callbacks: this.buildCallbackHandler(
          sessionId,
          catalogModel.stringifyEntityRef(userEntityRef)
        ),
        configurable: {
          thread_id: sessionId,
          credentials: options.credentials
        }
      }
    );
    const outputMessages = finalState.messages.slice(messages$1.length);
    if (outputMessages.length > 0) {
      const outputMessage = outputMessages[outputMessages.length - 1];
      output = outputMessage.content;
    } else {
      logger.error(`No output messages found for session ${sessionId}`);
      throw new Error(`No output messages found for session ${sessionId}`);
    }
    return {
      output
    };
  }
  buildSystemPrompt(template, userEntityRef) {
    return new messages.SystemMessage(
      template.replace("{username}", userEntityRef.name)
    );
  }
}

exports.LangGraphReactAgentType = LangGraphReactAgentType;
//# sourceMappingURL=LangGraphReactAgentType.cjs.js.map
